
#Include Once "windows.bi"
#Include Once "crt.bi"

Type BITMAPINFO2
    bmiHeader As BITMAPINFOHEADER
    bmiColors(0 To 255) As RGBQUAD
End Type

Extern GamePalette(0 To 255) Alias "GAMEPALETTE" As RGBQUAD

Dim Shared bmp As BITMAPINFO2
Dim Shared hInst As HINSTANCE
Dim Shared hWnd As HWND
Dim Shared msg As MSG
Dim Shared AppTitle As ZString*24 => "The Last Mission"
Dim Shared ScreenBigBuffer(0 To 640*400-1) as Ubyte
Dim Shared scale2x as Integer = 2

Declare Sub WndCreate
Declare Sub LM_GFX_Init
Declare Function LM_PollEvents As Byte

''================================================================================================
Function LM_Timer As Integer
    Return GetTickCount()
End Function

Sub LM_Sleep(sleep_time As Integer)
    'LM_PollEvents()
    Sleep_(sleep_time)
End Sub

Sub LM_Init
    WndCreate()
    LM_GFX_Init
End Sub

Sub LM_Deinit
    DestroyWindow(hWnd)
End Sub

Function LM_PollEvents As Byte

    If PeekMessage(@msg, NULL, 0, 0, PM_REMOVE) Then
      TranslateMessage(@msg)
      DispatchMessage(@msg)
      If msg.message = WM_QUIT Then Return 1
    End If

    Return 0
End Function

Function WndProc (ByVal hWnd As HWND, _
                  ByVal uMsg As UINT, _
                  ByVal wParam As WPARAM, _
                  ByVal lParam As LPARAM ) As LRESULT
    
    Select Case uMsg
        Case WM_CREATE
        Case WM_DESTROY
            PostQuitMessage(0)
        Case WM_CLOSE          
            LM_Deinit()
            ExitProcess(0)
        Case WM_KEYUP
            Keys((lParam Shr 16) And &h7f) = 0
        Case WM_KEYDOWN
            Keys((lParam Shr 16) And &h7f) = 1	        	
    End Select

    Return DefWindowProc(hWnd, uMsg, wParam, lParam)
End Function

Sub WndCreate
    Dim wc As WNDCLASS
    Dim rect As RECT

    With wc
        .style          = CS_HREDRAW Or CS_VREDRAW
        .lpfnWndProc    = @WndProc
        .cbClsExtra     = 0
        .cbWndExtra     = 0
        .hInstance      = hInst
        .hIcon          = NULL
        .hCursor        = LoadCursor(NULL, IDC_ARROW)
   		.hbrBackground  = Cast(HBRUSH, COLOR_WINDOW) '' to stop FBC from type whining
   		.lpszMenuName   = NULL
   		.lpszClassName  = @AppTitle
   	End With

    rect.top = 0
    rect.bottom = 400
    rect.left = 0
    rect.right = 640
   	
	If(RegisterClass(@wc) <> 0) Then
        AdjustWindowRect(@rect, WS_OVERLAPPED Or WS_CAPTION Or WS_SYSMENU Or WS_MINIMIZEBOX, NULL)

        hWnd = CreateWindowEx(0, @AppTitle, @AppTitle, _
            WS_OVERLAPPED Or WS_CAPTION Or WS_SYSMENU Or WS_MINIMIZEBOX, _
            CW_USEDEFAULT, CW_USEDEFAULT, _
            rect.right - rect.left, rect.bottom - rect.top, _
            NULL, NULL, hInst, NULL)
        If hWnd <> 0 Then 
            ShowWindow(hWnd, SW_SHOW)
            UpdateWindow(hWnd)
        End If
    End If 
End Sub

Sub LM_GFX_Init
    If GetDC(hWnd) = 0 Then Return    
    ReleaseDC(hWnd, 0)
    
    'memset(@bmp, SizeOf(bmp), 0)
    Asm
        mov edi, Offset bmp
        mov ecx, SizeOf(BITMAPINFO2)
        Xor eax, eax
        cld
        rep stosb
    End Asm
    
    bmp.bmiHeader.biSize = SizeOf(BITMAPINFOHEADER)
    bmp.bmiHeader.biWidth = 320 * scale2x
    bmp.bmiHeader.biHeight = -200 * scale2x
    bmp.bmiHeader.biPlanes = 1
    bmp.bmiHeader.biBitCount = 8
    bmp.bmiHeader.biCompression = BI_RGB
    
    'memcpy(@bmp.bmiColors(0), @GamePalette(0), 1024)
    Asm
        mov edi, Offset (bmp + OffsetOf(BITMAPINFO2, bmiColors(0)))
        mov esi, Offset GamePalette
        mov ecx, 1024/4
        rep movsd
    End Asm     
End Sub

Sub LM_GFX_Deinit
End Sub

Sub LM_GFX_Flip(p As UByte Ptr)
    Dim client As RECT
    Dim hDC As HDC
    
    hDC = GetDC(hWnd)
    If hDC <> 0 Then

        ' double 320x200 to 640x200
        if scale2x = 2 then
#if 0
        For y As Integer = 0 To 199
            For x As Integer = 0 To 319
                Dim As UByte tmp = Peek(p + y*320+x)

                ScreenBigBuffer((y*2+1)*640+x*2) = tmp
                ScreenBigBuffer((y*2+1)*640+x*2+1) = tmp
                ' dirty scanlines :)
                If tmp <= 8 andalso tmp <> 0 Then tmp += 8
                ScreenBigBuffer(y*2*640+x*2) = tmp
                ScreenBigBuffer(y*2*640+x*2+1) = tmp
            Next
        Next
#endif
#if 1
        asm
            push esi
            push edi
            
            mov  esi, [p]
            mov  edi, offset ScreenBigBuffer
            xor  ecx, ecx      
        __loopy:
            push ecx
            xor  ecx, ecx
        __loopx: 
            mov  al, byte ptr [esi+ecx]
            mov  ah, al
            mov word ptr [edi+ecx*2], ax
            or  al, al
            jz  __zero
            cmp al, 8
            ja  __zero
            add  ax, 0x0808
        __zero:
            mov word ptr [edi+ecx*2+640], ax
            inc  ecx
            cmp  ecx, 320
            jb   __loopx
            pop  ecx

            add esi, 320
            add edi, 640*2
            inc  ecx
            cmp  ecx, 200
            jb   __loopy

            pop  edi
            pop  esi
        end asm
#endif
        end if

        GetClientRect(hWnd, @client)
        'client.right = 640
        'client.bottom = 400
        StretchDIBits(hDC, _
                      0, _                    '' Destination top left hand corner X Position
                      0, _                    '' Destination top left hand corner Y Position
                      client.right, _         '' Destinations Width
                      client.bottom, _        '' Desitnations height
                      0, _                    '' Source top left hand corner's X Position
                      0, _                    '' Source top left hand corner's Y Position
                      320 * scale2x, _                  '' Sources width
                      200 * scale2x, _                  '' Sources height
                      iif(scale2x = 1, p, @ScreenBigBuffer(0)), _  '' Source's data
                      Cast(BITMAPINFO Ptr, @bmp), _ '' Bitmap Info
                      DIB_RGB_COLORS, _       '' operations
                      SRCCOPY)

                 
        ReleaseDC(hWnd, hDC)
    End If    
End Sub 

Sub LM_GFX_WaitVSync
    'Sleep_ 1000 \ 60
End Sub


Sub LM_GFX_SetScale(byval param as integer)
    scale2x = param
    bmp.bmiHeader.biWidth = 320 * param
    bmp.bmiHeader.biHeight = -200 * param
end sub

